package core;

import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

public class PresetParser {
	private File file;
	private Preset preset;
	
	public PresetParser( File file ) {
		this.file = file;
	}
	
	public Preset getPreset() {
		return preset;
	}
	
	public Preset parse() {
		String contents = readFile();
		preset = parsePreset( contents );
		return getPreset();
	}
	
	private String readFile() {
		StringBuffer sb = new StringBuffer();
		
		BufferedReader reader = null;
		try {
			reader = new BufferedReader( new FileReader( file ) );
			
			String line = null;
			while( null != ( line = reader.readLine() ) )
				sb.append( line ).append( "\r\n" );
		} catch( FileNotFoundException fnfe ) {
			throw new ParseException( fnfe );
		} catch( IOException ioe ) {
			throw new ParseException( ioe );
		}
		
		return sb.toString();
	}
	
	private Preset parsePreset( String string ) {
		Preset preset = new Preset();
		
		Map<Segment,String> segments = getSegments( string );
		for( Segment segment : segments.keySet() )
			switch( segment ) {
			
			case PHOTOLIST:
				String[] images = segments.get( segment ).split( "\\r\\n" );
				for( String img : images ) {
					File image = getFile( img, false );
					preset.addImage( image );
				}
				break;
				
			case TIMEPERPIC:
				try {
					preset.setTimePerPic( Integer.parseInt( segments.get( segment ) ) );
				} catch( NumberFormatException nfe ) {
					throw new ParseException( "Couldn't parse time per pic " + segments.get( segment ), nfe );
				}
				break;
				
			case FULLSCREEN:
				preset.setFullScreen( "Yes".equalsIgnoreCase( segments.get( segment ) ) );
				break;
				
			case AUDIO:
				preset.setAudio( getFile( segments.get( segment ), true ) );
				break;
			
			case PRESET_NAME:
				preset.setName( segments.get( segment ) );
				break;
				
			case AUTHOR:
				preset.setAuthor( segments.get( segment ) );
				break;
				
			case DESCRIPTION:
				preset.setDescription( segments.get( segment ) );
				break;
				
			case EMAIL:
				preset.setEmail( segments.get( segment ) );
				break;
				
			case WEBSITE:
				preset.setWebsite( segments.get( segment ) );
				break;
				
			case AVATAR:
				File avatar = getFile( segments.get( segment ), false );
				if( avatar != null )
					preset.setAvatar( avatar );
				break;
				
			case MAXWINSIZE:
				String[] parts = segments.get( segment ).split( "x" );
				if( parts != null && parts.length > 1 ) {
					try {
						int width, height;
						width = Integer.parseInt( parts[0] );
						height = Integer.parseInt( parts[1] );
						preset.setMaxWinSize( new Dimension( width, height ) );
					} catch( NumberFormatException nfe ) {
						throw new ParseException( "Error parsing max win size : " + segments.get( segment ), nfe );
					}
				}
				break;
				
			default:
				throw new ParseException( "Unknown segment " + segment );
			
			}
		
		return preset;
	}
	
	private Map<Segment,String> getSegments( String string ) {
		HashMap<Segment,String> segments = new HashMap<Segment,String>();
		for( Segment segment : Segment.values() ) {
			int start = segment.indexOfSegmentInString( string );
			if( start >= 0 ) {
				int end = Segment.indexOfAnySegmentInString( string, start ) - 1;
				String part = string.substring( start, end );
				String content = part.split( "\\r\\n", 2 )[1].trim();
				if( content.endsWith( "./." ) ) 
					content = content.substring( 0, content.lastIndexOf( "./." ) ).trim();
				if( !"".equals( content ) )
					segments.put( segment, content );
			}
		}
		return segments;
	}
	
	private File getFile( String name, boolean nullInsteadOfException ) {
		File res = getReadableFileOrNull( name );
		if( res != null )
			return res;
		
		for( String alternate : getPossibleAlternateFileNames( name ) )
			if( null != ( res = getReadableFileOrNull( alternate ) ) )
				return res;
		
		if( nullInsteadOfException )
			return null;
		throw new ParseException( "Can't find file " + new File( file.getParent(), name ).getAbsolutePath() );
	}
	private File getReadableFileOrNull( String name ) {
		File res = new File( file.getParent(), name );
		if( !res.exists() || !res.canRead() )
			return null;
		return res;
	}
	private List<String> getPossibleAlternateFileNames( String name ) {
		List<String> res = new ArrayList<String>();
		
		//if it looks like it might use backslash as dir seperators, try forward slashes
		if( name.contains( "\\" ) )
			res.add( name.replaceAll( "\\\\", "/" ) );
		
		int extensionSeperator = name.lastIndexOf( '.' );
		if( extensionSeperator < 0 )
			return res;
		
		String extension = name.substring( extensionSeperator+1, name.length() );
		String basename = name.substring( 0, extensionSeperator );
		
		//Try some different casing...
		Collection<String> basenames = new HashSet<String>();
		basenames.add( basename );
		basenames.add( basename.toLowerCase() );
		basenames.add( basename.toUpperCase() );
		if( basename.length() > 1 )
			basenames.add( basename.substring( 0, 1 ).toUpperCase() + basename.substring( 1, basename.length() ).toLowerCase() );
		
		Collection<String> extensions = new HashSet<String>();
		extensions.add( extension );
		extensions.add( extension.toLowerCase() );
		extensions.add( extension.toUpperCase() );
		if( extension.length() > 1 )
			extensions.add( extension.substring( 0, 1 ).toUpperCase() + extension.substring( 1, extension.length() ).toLowerCase() );
		
		for( String b : basenames )
			for( String e : extensions )
				res.add( b + "." + e );
		
		return res;
	}
	
	private enum Segment {
		AUDIO, 
		AUTHOR, 
		AVATAR, 
		DESCRIPTION, 
		EMAIL, 
		FULLSCREEN, 
		MAXWINSIZE, 
		PHOTOLIST, 
		PRESET_NAME, 
		TIMEPERPIC, 
		WEBSITE,
		;
		
		public int indexOfSegmentInString( String string, int from ) {
			return string.toLowerCase().indexOf( "["+name().toLowerCase()+"]", from );
		}
		public int indexOfSegmentInString( String string ) {
			return indexOfSegmentInString( string, 0 );
		}
		
		public static int indexOfAnySegmentInString( String string, int from ) {
			int lowestIndex = string.length();
			for( Segment segment : values() ) {
				int index = segment.indexOfSegmentInString( string, from );
				if( index <= lowestIndex && index > from )
					lowestIndex = index;
			}
			return lowestIndex;
		}
	}
	
	public static class ParseException extends RuntimeException {
		private static final long serialVersionUID = 1L;
		
		public ParseException( Exception e ) {
			super( e );
		}
		public ParseException( String msg, Exception e ) {
			super( msg, e );
		}
		public ParseException( String msg ) {
			super( msg );
		}
	}
}
