package mainview;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JOptionPane;

import presetview.PresetView;
import core.Engine;
import core.Preset;

public class Actions {
	private MainView mainView;
	private PresetFileChooser fileChooser;
	
	public Actions( MainView mainView ) {
		this.mainView = mainView;
		fileChooser = new PresetFileChooser( mainView );
	}
	
	public abstract class AbstractMainViewAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		public AbstractMainViewAction( String name, Integer mnemonic ) {
			super( name );
			if( mnemonic != null )
				putValue( MNEMONIC_KEY, mnemonic );
		}
		public void perform() {
			actionPerformed( null );
		}
	}
	
	public abstract class BooleanAction extends AbstractMainViewAction {
		private static final long serialVersionUID = 1L;
		public BooleanAction( String name, Integer mnemonic ) {
			super( name, mnemonic );
		}
		protected boolean getState( ActionEvent event ) {
			return ( ( JCheckBoxMenuItem ) event.getSource() ).getState();
		}
		protected abstract boolean getState();
	}
	
	public abstract class IntegerAction extends AbstractMainViewAction {
		private static final long serialVersionUID = 1L;
		public IntegerAction( String name, Integer mnemonic ) {
			super( name, mnemonic );
		}
		protected Integer getInteger( String title, String message, Object initialValue ) {
			Object res = JOptionPane.showInputDialog( mainView, message, title, JOptionPane.QUESTION_MESSAGE, null, null, initialValue );
			if( res == null )
				return null;
			if( res instanceof Integer )
				return ( Integer ) res;
			if( res instanceof String )
				try {
					return Integer.parseInt( ( String ) res );
				} catch( NumberFormatException nfe ) {
					return getInteger( title, message,res );
				}
			return getInteger( title, message, res );
		}
		public abstract void setInteger( Integer integer );
	}
	
	public class MessageAction extends AbstractMainViewAction {
		private static final long serialVersionUID = 1L;
		private String message;
		public MessageAction( String name, Integer mnemonic, String message ) {
			super( name, mnemonic );
			this.message = message;
		}
		@Override
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog( mainView, message );
		}
		
	}
	
	public AbstractMainViewAction openPresetAction = new AbstractMainViewAction( "Open Preset", KeyEvent.VK_O ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed(ActionEvent arg0) {
			File f = fileChooser.open();
			if( f != null ) {
				Engine.setCurrentPreset( f );
				Preset p = Engine.getCurrentPreset();
				if( p != null ) {
					mainView.setPreset( p );
					runPresetAction.setEnabled( true );
					if( Engine.getSettings().isAutorun() )
						runPresetAction.perform();
				}
			}
		}
	};
	
	public AbstractMainViewAction exitAction = new AbstractMainViewAction( "Exit", KeyEvent.VK_X ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed(ActionEvent arg0) {
			mainView.close();
		}
	};
	
	public BooleanAction forceFullScreenAction = new BooleanAction( "Force Fullscreen", KeyEvent.VK_F ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed( ActionEvent event ) {
			Engine.getSettings().setForceFullscreen( getState( event ) ); 
		}
		public boolean getState() {
			return Engine.getSettings().isForceFullscreen();		
		}
	};
	
	public BooleanAction stretchImagesAction = new BooleanAction( "Stretch Images", KeyEvent.VK_S ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed( ActionEvent event ) {
			Engine.getSettings().setStretchImages( getState( event ) ); 
		}
		public boolean getState() {
			return Engine.getSettings().isStretchImages();		
		}
	};
	
	public AbstractMainViewAction backgroundAction = new AbstractMainViewAction( "Set BG Color", KeyEvent.VK_B ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed( ActionEvent event ) {
			Color newBackground = JColorChooser.showDialog( mainView, "Color", Engine.getSettings().getBackground() );
			if( newBackground != null )
				Engine.getSettings().setBackground( newBackground );
		}
	};
	
	public BooleanAction audioAction = new BooleanAction( "Audio Enabled", KeyEvent.VK_A ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed( ActionEvent event ) {
			Engine.getSettings().setAudio( getState( event ) ); 
		}
		public boolean getState() {
			return Engine.getSettings().isAudio();		
		}
	};
	
	public IntegerAction maxNrImagesAction = new IntegerAction( "Set Max # of Photos", KeyEvent.VK_M ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed(ActionEvent arg0) {
			Integer value = getInteger( "Set Max. Number Of Photos", "Please set the maximum number of photos the program will allow presets to have:", Engine.getSettings().getMaxNrOfImages() );
			setInteger( value );
		}	
		public void setInteger( Integer integer ) {
			if( integer != null && integer > 0 )
				Engine.getSettings().setMaxNrOfImages( integer );
		}
	};
	
	public IntegerAction minTimePerPicAction = new IntegerAction( "Set Min. Time per Photo", KeyEvent.VK_M ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed(ActionEvent arg0) {
			Integer value = getInteger( "Set Min. Time per Photo", "Please set the minimum number of milliseconds each photo should be displayed:", Engine.getSettings().getMinTimePerPic() );
			setInteger( value );
		}	
		public void setInteger( Integer integer ) {
			if( integer != null && integer > 0 )
				Engine.getSettings().setMinTimePerPic( integer );
		}
	};
	
	public IntegerAction durationAction = new IntegerAction( "Set duration", KeyEvent.VK_D ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed(ActionEvent arg0) {
			Integer value = getInteger( "Set duration", "Please set the duration a preset should be displayed (in minutes) :", Engine.getSettings().getDuration() );
			setInteger( value );
		}	
		public void setInteger( Integer integer ) {
			if( integer != null && integer > 0 )
				Engine.getSettings().setDuration( integer );
		}
	};
	
	public BooleanAction autorunAction = new BooleanAction( "Auto-run opened files", KeyEvent.VK_R ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed( ActionEvent event ) {
			Engine.getSettings().setAutorun( getState( event ) ); 
		}
		public boolean getState() {
			return Engine.getSettings().isAutorun();		
		}
	};
	
	public MessageAction helpAction = new MessageAction( "Help", KeyEvent.VK_H, 
			"Use File -> Open Preset and select an *.svp file, then click 'Run Preset'."
	);
	
	public MessageAction aboutAction = new MessageAction( "About", KeyEvent.VK_A, 
			"Original SubViewer created by SYNeR.  Rewritten in Java by kimmyfahrad@yahoo.com.  Enjoy!"
	);
	
	public AbstractMainViewAction runPresetAction = new AbstractMainViewAction( "Run Preset", KeyEvent.VK_R ) {
		private static final long serialVersionUID = 1L;
		{
			setEnabled( false );//enabled by openPresetAction after a preset is loaded
		}
		@Override
		public void actionPerformed(ActionEvent arg0) {
			new PresetView( Engine.getCurrentPreset() );
		}
	};
	
	public AbstractMainViewAction resetSettingsAction = new AbstractMainViewAction( "Reset settings", KeyEvent.VK_S ) {
		private static final long serialVersionUID = 1L;
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if( JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog( mainView, "Are you sure you wish to reset the settings back to their default values?", "Reset settings", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE ) ) {
				Engine.resetSettings();
				mainView.repaint();
			}
		}
	};
}
