package presetview;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import core.Engine;
import core.Preset;

public class PresetImagePanel extends JPanel {
	private static final long serialVersionUID = 1L;
	
	private List<File> imageFiles;
	private int timePerPic;
	private Color backgroundColor;
	private boolean running = false;
	private ImageSwitcher imageSwitcher;
	
	public PresetImagePanel( Preset preset ) {
		this.imageFiles = preset.getImages();
		int maxNrOfImages = Engine.getSettings().getMaxNrOfImages();
		if( maxNrOfImages > 0 && maxNrOfImages < this.imageFiles.size() )
			this.imageFiles = this.imageFiles.subList( 0, maxNrOfImages );
		this.timePerPic = Math.max( preset.getTimePerPic(), Engine.getSettings().getMinTimePerPic() );
		this.backgroundColor = Engine.getSettings().getBackground();
		imageSwitcher = new ImageSwitcher();
	}
	
	public void start() {
		running = true;
		if( !imageSwitcher.isAlive() )
			imageSwitcher.start();
	}
	
	public void stop() {
		running = false;
		if( imageSwitcher != null )
			imageSwitcher.deactivate();
	}
	
	public void paintComponent( Graphics g ) {
		int width = getWidth();
		int height = getHeight();
		g.setColor( backgroundColor );
		g.fillRect( 0, 0, width, height );
		if( imageSwitcher != null && running ) {
			BufferedImage image = imageSwitcher.getCurrentImage();
			int imageWidth = image.getWidth();
			int imageHeight = image.getHeight();
			if( Engine.getSettings().isStretchImages() ) {
				double scaleFactor = Math.min( (double) width / (double) imageWidth, (double) height / (double) imageHeight );
				imageWidth *= scaleFactor;
				imageHeight *= scaleFactor;
			}
			g.drawImage( image, ( width - imageWidth ) / 2, ( height - imageHeight ) / 2, imageWidth, imageHeight, null );
		}
	}
	
	private class ImageSwitcher extends Thread {
		private Map<Integer,BufferedImage> images = new HashMap<Integer,BufferedImage>();
		private int currentImage = -1;
		private long lastImageSwitch = 0;
		private boolean active = true;
		
		public ImageSwitcher() {
			updateCurrentImage();//make sure image 0 is loaded as that will be shown by default
			preload();//start preloading images in a seperate thread
		}
		
		public void run() {
			while( running && active ) {
				if( shouldUpdate() )
					update();
				yield();
			}
		}
		
		public BufferedImage getCurrentImage() {
			return images.get( currentImage );
		}
		
		private void update() {
			lastImageSwitch = System.currentTimeMillis();
			updateCurrentImage();
			repaint();
		}
		
		private void updateCurrentImage() {
			int newCurrentImage = ( currentImage + 1 ) % imageFiles.size();
			loadImageIfNeeded( newCurrentImage );
			currentImage = newCurrentImage;
		}
		
		private void loadImageIfNeeded( int i ) {
			if( !images.containsKey( i ) ) 
                            loadImage( i );
		}

                private void loadImage( int i ) {
                    try {
                            images.put( i, ImageIO.read( imageFiles.get( i ) ) );
                    } catch (IOException e) {
                            System.out.println( "Error reading image " + imageFiles.get( i ).getAbsolutePath() + " : " + e.getMessage() );
                            e.printStackTrace();
                    }
                }
		
		private boolean shouldUpdate() {
			return System.currentTimeMillis() - lastImageSwitch >= timePerPic;
		}
		
		public void deactivate() {
			active = false;
		}
		
		private void preload() {
			new Thread() {
				public void run() {
					for( int i=0; i<imageFiles.size() && active; i++ ) {
						loadImageIfNeeded( i );
						yield();
					}
				}
			}.start();
		}
	}
}
