package settings;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

public class Settings {
	private static final String TRUE = "True", FALSE = "False";
	private static final String FILENAME = "settings.dat";
	private static final String SEPERATOR = "\r\n";
	
	//SubViewer settings
	private boolean forceFullscreen = false;
	private boolean stretchImages = false;
	private Color background = Color.BLACK;
	private int dummyUnknownMaybeColorRelated1 = 0, dummyUnknownMaybeColorRelated2 = 0;
	private boolean dummyShowCloseButton = false;
	private int duration = 15;
	private int maxNrOfImages = 0;
	private boolean autorun = false;
	//Extra settings not present in the original.  SubViewer will remove these when it overwrites the file
	private int minTimePerPic = 0;
	private boolean audio = true;
	private String lastFileLocation = ".";
	
	public boolean isForceFullscreen() {
		return forceFullscreen;
	}
	public void setForceFullscreen(boolean forceFullscreen) {
		this.forceFullscreen = forceFullscreen;
	}
	
	public boolean isStretchImages() {
		return stretchImages;
	}
	public void setStretchImages(boolean stretchImages) {
		this.stretchImages = stretchImages;
	}
	
	public Color getBackground() {
		return background;
	}
	public void setBackground(Color background) {
		this.background = background;
	}
	
	public int getDuration() {
		return duration;
	}
	public void setDuration(int duration) {
		this.duration = duration;
	}
	
	public int getMaxNrOfImages() {
		return maxNrOfImages;
	}
	public void setMaxNrOfImages(int maxNrOfImages) {
		this.maxNrOfImages = maxNrOfImages;
	}
	
	public boolean isAutorun() {
		return autorun;
	}
	public void setAutorun(boolean autorun) {
		this.autorun = autorun;
	}
	
	public int getMinTimePerPic() {
		return minTimePerPic;
	}
	public void setMinTimePerPic(int minTimePerPic) {
		this.minTimePerPic = minTimePerPic;
	}
	
	public boolean isAudio() {
		return audio;
	}
	public void setAudio(boolean audio) {
		this.audio = audio;
	}
	
	public String getLastFileLocation() {
		return lastFileLocation;
	}
	public void setLastFileLocation(String lastFileLocation) {
		this.lastFileLocation = lastFileLocation;
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append( forceFullscreen ? TRUE : FALSE ).append( SEPERATOR );
		sb.append( stretchImages ? TRUE : FALSE ).append( SEPERATOR );
		sb.append( background == null ? "" : fixRGB( background.getRGB() ) ).append( SEPERATOR );
		sb.append( dummyUnknownMaybeColorRelated1 ).append( SEPERATOR );
		sb.append( dummyUnknownMaybeColorRelated2 ).append( SEPERATOR );
		sb.append( dummyShowCloseButton ? TRUE : FALSE ).append( SEPERATOR );
		sb.append( duration ).append( SEPERATOR );
		sb.append( maxNrOfImages ).append( SEPERATOR );
		sb.append( autorun ? TRUE : FALSE ).append( SEPERATOR );
		sb.append( minTimePerPic ).append( SEPERATOR );
		sb.append( audio ? TRUE : FALSE ).append( SEPERATOR );
		sb.append( lastFileLocation != null ? lastFileLocation : "" ).append( SEPERATOR );
		return sb.toString();
	}
	
	public static Settings read() {
		Settings settings = new Settings();
		
		File file = new File( FILENAME );
		if( file.exists() && file.canRead() ) {
			try {
				BufferedReader reader = new BufferedReader( new FileReader( file ) );
				
				settings.forceFullscreen = readBoolean( reader, settings.forceFullscreen );
				settings.stretchImages = readBoolean( reader, settings.stretchImages );
				settings.background = readColor( reader, settings.background );
				settings.dummyUnknownMaybeColorRelated1 = readInteger( reader, settings.dummyUnknownMaybeColorRelated1 );
				settings.dummyUnknownMaybeColorRelated2 = readInteger( reader, settings.dummyUnknownMaybeColorRelated2 );
				settings.dummyShowCloseButton = readBoolean( reader, settings.dummyShowCloseButton );
				settings.duration = readInteger( reader, settings.duration );
				settings.maxNrOfImages = readInteger( reader, settings.maxNrOfImages );
				settings.autorun = readBoolean( reader, settings.autorun );
				settings.minTimePerPic = readInteger( reader, settings.minTimePerPic );
				settings.audio = readBoolean( reader, settings.audio );
				settings.lastFileLocation = readString( reader, settings.lastFileLocation );
			} catch( FileNotFoundException fnfe ) {
				fnfe.printStackTrace();
			} catch( IOException ioe ) {
				ioe.printStackTrace();
			}
		}
		
		return settings;
	}
	private static boolean readBoolean( BufferedReader reader, boolean defaultValue ) throws IOException {
		String line = readLine( reader );
		if( !"".equals( line ) )
			return TRUE.equals( line );
		return defaultValue;
	}
	private static int readInteger( BufferedReader reader, int defaultValue ) throws IOException {
		String line = readLine( reader );
		if( !"".equals( line ) )
			try {
				return Integer.parseInt( line );
			} catch( NumberFormatException nfe ) {
				nfe.printStackTrace();
			}
		return defaultValue;
	}
	private static Color readColor( BufferedReader reader, Color defaultValue ) throws IOException {
		String line = readLine( reader );
		if( !"".equals( line ) )
			try {
				int rgb = Integer.parseInt( line );
				return new Color( fixRGB( rgb ) );
			} catch( NumberFormatException nfe ) {
				nfe.printStackTrace();
			}
		return defaultValue;
	}
	private static String readString( BufferedReader reader, String defaultValue ) throws IOException {
		String line = readLine( reader );
		if( !"".equals( line ) )
			return line;
		return defaultValue;
	}
	private static String readLine( BufferedReader reader ) throws IOException {
		String line = reader.readLine();
		return line == null ? "" : line.trim();
	}
	
	private static int fixRGB( int rgb ) {
		rgb &= 0xffffff;//strip most significant bits, these contain alpha values in Java and nothing relevant in SubViewer's language
		//then swap the 2nd and 4th bytes because Java uses RGB while SubViewer's language uses BGR
		return ((rgb >> 16) & 0xff) + (rgb & 0xff00) + ((rgb & 0xff) << 16);
	}
	
	public static void write( Settings settings ) {
		try {
			BufferedWriter writer = new BufferedWriter( new FileWriter( new File( FILENAME ) ) );
			writer.write( settings.toString() );
			writer.flush();
			writer.close();
		} catch( FileNotFoundException fnfe ) {
			fnfe.printStackTrace();
		} catch( IOException ioe ) {
			ioe.printStackTrace();
		}
	}
}
